/*****************************************************
 *                 TINY DJ SKETCH                    *
 *****************************************************/
#include <SD.h>
#include <SPI.h>
#include <TMRpcm.h> 
#define SD_CARD 10  
#define SD_ChipSelectPin 10

TMRpcm music;
int musicSwitch = 3;  // Little red button
int ledSwitch = 2;    // Little green button
int redPin = 4;
int greenPin = 5;
int bluePin = 6;
int motorPin = 7;
int color = 0;
long delayTime = 213.13;
int randMusic;


/*****************************************************
 *                    SETUP                          *
 *****************************************************/
void setup() {
  pinMode(musicSwitch, INPUT);
  pinMode(ledSwitch, INPUT);
  pinMode(redPin, OUTPUT);
  pinMode(greenPin, OUTPUT);
  pinMode(bluePin, OUTPUT); 
  pinMode(motorPin, OUTPUT);
  
  music.speakerPin = 9;
  Serial.begin(9600);
  
  // See if the card is present and can be initialized:
  if (!SD.begin(SD_ChipSelectPin)) {  
    Serial.println("SD fail");  
    return;   // don't do anything more if not
  }
  
  randomSeed(analogRead(0));
}


/*****************************************************
 *                      LOOP                         *
 *****************************************************/
void loop() {
  digitalWrite(motorPin, LOW);
  music.quality(1);
  music.setVolume(5);
  delayTime = analogRead(A0)/2;
  
  Serial.print(analogRead(A0));
  Serial.print("\n");
  
  // Left button : play a random music of the SD card
  if (digitalRead(musicSwitch) == HIGH) {
    randMusic = random(3);
    playRandMusic(randMusic);
  }

  // General functioning when music is on 
  if (music.isPlaying()) {
    digitalWrite(motorPin, HIGH);
    changeColor(color, delayTime);
    
    // Right button : Change the blink color
    if (digitalRead(ledSwitch) == HIGH) {
      color = (color + 1) % 5;
    } 

  // When the song is finished, turn off the motor
  } else {
    digitalWrite(motorPin, LOW);
  }
}


/*****************************************************
 *               AUXILIARY FONCTIONS                 *
 *****************************************************/
/*
 * Play a random music from the SD card (from 0 to 3).
 */
void playRandMusic(int number) {
    switch (number) {
      case 0 :
        music.play("0.wav");
        break;
      case 1 :
        music.play("1.wav");
        break;
      case 2 :
        music.play("2.wav");
        break;
      case 3 :
        music.play("3.wav");
        break;
      default :
        break;
    }
}

/*
 * Set the color of the LED lamp.
 * In our case, we substract the color to 255.
 */
void setColor(int red, int green, int blue)
{
  #ifdef COMMON_ANODE
    red = 255 - red;
    green = 255 - green;
    blue = 255 - blue;
  #endif
  analogWrite(redPin, red);
  analogWrite(greenPin, green);
  analogWrite(bluePin, blue); 
}

/*
 * Alternate 4 colors for the LED.
 */
void djLights(long delayT) {
  setColor(0, 255, 255);  // red
  delay(delayT);
  setColor(255, 255, 255);  // green
  delay(delayT);
  setColor(0, 255, 0);  // purple
  delay(delayT);
  setColor(255, 255, 0);  // blue
  delay(delayT);
  setColor(0, 0, 0); // no color
}

/*
 * Blink with a single color defined by the parameters.
 */
void monoBlink(int r, int g, int b, long delayT) {
  setColor(r, g, b);
  delay(delayT);
  setColor(0, 0, 0);
  delay(delayT);
}

/*
 * Set the blinking color of the LED.
 * If 0 : DJ lights ; from 1 to 4 : mono color.
 */
void changeColor(int color, long delayT) {
  switch (color) {
    case 0 :
      djLights(delayT);
      break;
    case 1 :
      monoBlink(0, 255, 255, delayT);  // red
      break;
    case 2 :
      monoBlink(255, 255, 255, delayT);  // green
      break;
    case 3 :
      monoBlink(0, 255, 0, delayT);  // purple
      break;
    case 4 :
      monoBlink(255, 255, 0, delayT);  // blue
      break;
    default :
      break;
  }
}

